require 'faraday'
require 'faraday_middleware'
require 'oj'

module Oma
  # Client implementation
  module Ruby
    # Client class
    class Client
      attr_reader :url, :user, :password

      include Errors
      include Actions

      def initialize(url:, user:, password:)
        @url      = url
        @user     = user
        @password = password
      end

      API_ACTIONS.each do |action|
        define_method(action) do |params = nil|
          request(action, params || {})
        end
      end

      private

      def api
        @api ||= Faraday.new(url: url) do |connection|
          connection.request :json
          connection.adapter Faraday.default_adapter
        end
      end

      def request(action, params)
        response = api.post(action, format_params(params))

        handle_reponse(response)
      end

      def handle_reponse(response)
        check_http_error!(response)
        check_api_error!(response)

        Oj.load(response.body)
      end

      def check_http_error!(response)
        return if response.status == 200

        raise http_error(response.status)
      end

      def check_api_error!(response)
        body = Oj.load(response.body)

        return if body['success']

        raise api_error(body['error_number']), body['error']
      end

      def api_credentials
        @api_credentials ||= { user: user, password: password }
      end

      def format_params(params)
        { credentials: api_credentials }.merge(params)
      end
    end
  end
end
