module Oma
  # HTTP and API Errors mapping
  module Ruby
    # Errors module
    module Errors
      class HttpError < StandardError; end
      class OpenSRSApiError < StandardError; end

      HTTP_ERRORS = {
        '400' => 'BadRequestError',
        '404' => 'NotFoundError',
        '500' => 'InternalServerError'
      }.freeze

      API_ERRORS = {
        '0'  => 'ServerError',
        '1'  => 'InvalidCredentialsError',
        '2'  => 'RequestedObjectDoesNotExistError',
        '3'  => 'ObjectIsAliasError',
        '4'  => 'RequestorLacksPermissionError',
        '5'  => 'RequestBadlyFormattedError',
        '6'  => 'AttributesBadlyFormattedError',
        '7'  => 'ObjectAlreadyExistsError',
        '8'  => 'DomainDoesNotExistError',
        '9'  => 'RequestorDoesNotOwnObjectError',
        '10' => 'RequestedObjectNotEmptyError',
        '11' => 'CompanyDoesNotExistError',
        '12' => 'RoleDoesNotExistError',
        '13' => 'UserDoesNotExistError',
        '14' => 'BrandInUseError',
        '15' => 'DomainUsersFullError',
        '16' => 'DomainAliasesFullError',
        '17' => 'NotInError',
        '18' => 'WorkgroupIsDefaultError',
        '19' => 'MigrationJobExistsError',
        '20' => 'TryAgainLaterError'
      }.freeze

      HTTP_ERRORS.each_value do |error|
        const_set(error, Class.new(HttpError))
      end

      API_ERRORS.each_value do |error|
        const_set(error, Class.new(OpenSRSApiError))
      end

      def http_error(status)
        Oma::Ruby::Errors.const_get(HTTP_ERRORS.fetch(status.to_s, 'HttpError'))
      end

      def api_error(code)
        Oma::Ruby::Errors.const_get(API_ERRORS.fetch(code.to_s, 'OpenSRSApiError'))
      end
    end
  end
end
